import greenfoot.*;
import java.util.List;

/**
 * Окно основного поля игры.
 * 
 * @author Vladimir Gonchar, Roman Borisov, Sofiya Lichkovaha
 */
public class Battleship extends World
{
    private final int MIN_X = 4;
    private final int MAX_X = 13;
    private final int MIN_X_COMPUTER = 22;
    private final int MAX_X_COMPUTER = 31;
    private final int MIN_Y = 7;
    private final int MAX_Y = 16;
    private int initialHitX;
    private int initialHitY;
    private int hitX;
    private int hitY;
    private int x;
    private int y;
    private int streak;
    private int direction;
    private int switching;
    private Ship[] ships;
    private Ship[] computerShips;
    private Ship[] shipsIcons;
    private Ship[] computerShipsIcons;
    private boolean started;
    private boolean computerTurn = false;
    private boolean hit = false;
    private text textStart;
    private Crosshair player;
    
    public Battleship() {    
        super(36, 24, 20); // 720x480
        setPaintOrder(Crosshair.class, Guess.class, Ship.class, Block.class);
        player = new Crosshair();
        setup();        
    }
    
    public void act() {
        if (started) {
            if (!player.isPlayerTurn()) {
                endGame();
                computerTurn = true;
            }
            if (computerTurn && player.getWorld() != null) {
                guess();
                endGame();
            }
            player.setPlayerTurn(true);
            computerTurn = false;
        }
    }
    
    private void setup() {
        started = false;
        for (int x = 0; x <= (MAX_X - MIN_X); x++) {
            for (int y = 0; y <= (MAX_Y - MIN_Y); y++) {

                addObject(new PlayerBlock(), x + MIN_X, y + MIN_Y);

                addObject(new ComputerBlock(), x + MIN_X_COMPUTER, y + MIN_Y);
            }
        }

        
        addObject(new Start(), 27, 21);
        addObject(new Exit(), 29, 2);
          
        textStart = new text();
        addObject(textStart, 14, 2);

        ships = new Ship[8];
        ships[0] = new Ship2();
        ships[1] = new Ship2();
        ships[2] = new Ship3();
        ships[3] = new Ship3();
        ships[4] = new Ship4();
        ships[5] = new Ship4();
        ships[6] = new Ship5();
        ships[7] = new Ship5();

        addObject(ships[0], 5, 19);
        addObject(ships[1], 11, 19);
        addObject(ships[2], 5, 20);
        addObject(ships[3], 11, 20);
        addObject(ships[4], 6, 21);
        addObject(ships[5], 12, 21);
        addObject(ships[6], 6, 22);
        addObject(ships[7], 12, 22);

        shipsIcons = new Ship[8];
        shipsIcons[0] = new Ship2();
        shipsIcons[1] = new Ship2();
        shipsIcons[2] = new Ship3();
        shipsIcons[3] = new Ship3();
        shipsIcons[4] = new Ship4();
        shipsIcons[5] = new Ship4();
        shipsIcons[6] = new Ship5();
        shipsIcons[7] = new Ship5();

        for (int i = 0; i < ships.length; i++) {
            ships[i].setIcon(shipsIcons[i]);
        }

        computerShips = new Ship[8];
        computerShips[0] = new Ship2();
        computerShips[0].setImage("computership2.png");
        computerShips[1] = new Ship2();
        computerShips[1].setImage("computership2.png");
        computerShips[2] = new Ship3();
        computerShips[2].setImage("computership3.png");
        computerShips[3] = new Ship3();
        computerShips[3].setImage("computership3.png");
        computerShips[4] = new Ship4();
        computerShips[4].setImage("computership4.png");
        computerShips[5] = new Ship4();
        computerShips[5].setImage("computership4.png");
        computerShips[6] = new Ship5();
        computerShips[6].setImage("computership5.png");
        computerShips[7] = new Ship5();
        computerShips[7].setImage("computership5.png");

        for (Ship s : computerShips) {
            addObject(s, 20,18);
        }

        computerShipsIcons = new Ship[8];
        computerShipsIcons[0] = new Ship2();
        computerShipsIcons[1] = new Ship2();
        computerShipsIcons[2] = new Ship3();
        computerShipsIcons[3] = new Ship3();
        computerShipsIcons[4] = new Ship4();
        computerShipsIcons[5] = new Ship4();
        computerShipsIcons[6] = new Ship5();
        computerShipsIcons[7] = new Ship5();

        for (int i = 0; i < ships.length; i++) {
            computerShips[i].setIcon(computerShipsIcons[i]);
        }

        for (int i = computerShips.length - 1; i >= 0; i--) {

            if (Greenfoot.getRandomNumber(2) == 0) {
                computerShips[i].rotate();
            }

            int posX;
            int posY;
            do {
                posX = MIN_X_COMPUTER + Greenfoot.getRandomNumber(MAX_X_COMPUTER - MIN_X_COMPUTER + 1);
                posY = MIN_Y + Greenfoot.getRandomNumber(MAX_Y - MIN_Y + 1);
            } while (!computerShips[i].move(posX, posY));
        }
    }
    
    public boolean beginGame() {
        boolean canBegin = true;
        for (Ship ship : ships) {
            if (!ship.isInPosition()) {
                canBegin = false;
                break;
            }
        }
        if (canBegin) {
            started = true;
            addObject(player, MIN_X_COMPUTER, MIN_Y);
            removeObject(textStart);
            addIcons();
        }
        return canBegin;
    }
    
    private void addIcons() {
        
        addObject(ships[0], 5, 19);
        addObject(ships[1], 11, 19);
        addObject(ships[2], 5, 20);
        addObject(ships[3], 11, 20);
        addObject(ships[4], 6, 21);
        addObject(ships[5], 12, 21);
        addObject(ships[6], 6, 22);
        addObject(ships[7], 12, 22);
        
        addObject(shipsIcons[0], 5, 19);
        addObject(shipsIcons[1], 11, 19);
        addObject(shipsIcons[2], 5, 20);
        addObject(shipsIcons[3], 11, 20);
        addObject(shipsIcons[4], 6, 21);
        addObject(shipsIcons[5], 12, 21);
        addObject(shipsIcons[6], 6, 22);
        addObject(shipsIcons[7], 12, 22);
        addObject(computerShipsIcons[0], 23, 19);
        addObject(computerShipsIcons[1], 29, 19);
        addObject(computerShipsIcons[2], 23, 20);
        addObject(computerShipsIcons[3], 29, 20);
        addObject(computerShipsIcons[4], 24, 21);
        addObject(computerShipsIcons[5], 30, 21);
        addObject(computerShipsIcons[6], 24, 22);
        addObject(computerShipsIcons[7], 30, 22);
    }

    private void guess() {
        int count = 0;
        do {
            if (count >= 20) {
                count = 0;
                hit = false;
                streak = 0;
                switching = 0;
            }
            if (!hit && (switching >= 2 || streak >= 4)) {
                hit = true;
                hitX = initialHitX;
                hitY = initialHitY;
                streak = 1;
            }
            if (!hit) {
                switching = 0;
                streak = 0;
                int cycles = 0;
                int guesses = 0;
                PlayerBlock block;
                do {
 
                    if (cycles >= 40 && guesses < 4) {
                        guesses++;
                        cycles = 0;
                    }
                    do {
                        x = MIN_X + Greenfoot.getRandomNumber(MAX_X - MIN_X + 1);
                        y = MIN_Y + Greenfoot.getRandomNumber(MAX_Y - MIN_Y + 1);
                    } while (alreadyGuessed());
                    block = getObjectsAt(x, y, PlayerBlock.class).get(0);
                    cycles++;
                } while (block.getMissedAttacks() > guesses);
                initialHitX = x;
                initialHitY = y;
                cycles = 0;
            } else {

                if (streak == 1) {

                    do {
 
                        direction = Greenfoot.getRandomNumber(4);
                        count++;
                        switch (direction) {
                            case 0: 
                            x = hitX + 1;
                            y = hitY;
                            break;
                            case 1:
                            x = hitX;
                            y = hitY + 1;
                            break;
                            case 2:
                            x = hitX - 1;
                            y = hitY;
                            break;
                            case 3:
                            x = hitX;
                            y = hitY - 1;
                            break;
                        }
                    } while ((!inGrid() || alreadyGuessed()) && count < 20);
                } else {
                    
                    switch (direction) {
                        case 0: 
                        x = hitX + 1;
                        y = hitY;
                        break;
                        case 1:
                        x = hitX;
                        y = hitY + 1;
                        break;
                        case 2:
                        x = hitX - 1;
                        y = hitY;
                        break;
                        case 3:
                        x = hitX;
                        y = hitY - 1;
                        break;
                    }
                    if (!inGrid() || alreadyGuessed()) {

                        direction = (direction + 2) % 4;
                        switching ++;
                        switch (direction) {
                            case 0: 
                            x = hitX + streak;
                            y = hitY;
                            break;
                            case 1:
                            x = hitX;
                            y = hitY + streak;
                            break;
                            case 2:
                            x = hitX - streak;
                            y = hitY;
                            break;
                            case 3:
                            x = hitX;
                            y = hitY - streak;
                            break;
                        }
                        if (!inGrid() || alreadyGuessed()) {
                            streak = 1;
                            continue;
                        }
                    }
                }
            }
        } while (!inGrid() || alreadyGuessed());
        Ship ship = getShip();
        if (ship != null) {
            addObject(new Hit(), x, y);
            Greenfoot.delay(50);
            ship.sink();
            hitX = x;
            hitY = y;
            if (ship.isSunk()) {

                hit = false;
            } else {
                hit = true;
                streak++;
            }
        } else {
            addObject(new Miss(), x, y);
            Greenfoot.delay(50);
        }
    }
 
    private Ship getShip() {
        List<Ship> list = getObjectsAt(x, y, Ship.class);
        if (list.size() == 0) {
            return null;
        }
        Ship ship = list.get(0);
        if (ship == null) {
            return null;
        }
        if (ship.isBlank(x, y)) {
            return null;
        } else {
            return ship;
        }
    }
    
    private boolean inGrid() {
        return (x >= MIN_X && x <= MAX_X && y >= MIN_Y && y <= MAX_Y);
    }
    
    public boolean inComputerGrid(int x, int y) {
        return (x >= MIN_X_COMPUTER && x <= MAX_X_COMPUTER && y >= MIN_Y && y <= MAX_Y);
    }

    private boolean alreadyGuessed() {
        return (getObjectsAt(x, y, Guess.class).size() != 0);        
    }
    
    private boolean Bombclicked() {
        return (getObjectsAt(x, y, bomb.class).size() != 0);        
    }
    
    public boolean hasStarted() {
        return started;
    }
    
    private void endGame() {
        boolean isOver = true;
        for (Ship ship : computerShips) {
            if (!ship.isSunk()) {
                isOver = false;
                break;
            }
        }
        if (isOver) {
            removeObject(player);
            Greenfoot.setWorld(new WinFrame());
            return;
        }
        isOver = true;
        for (Ship ship : ships) {
            if (!ship.isSunk()) {
                isOver = false;
                break;
            }
        }
        if (isOver) {
            removeObject(player);
            Greenfoot.setWorld(new LoseFrame());
        }
    }
}
