import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Корабли, обработка
 * 
 * @author Vladimir Gonchar, Roman Borisov, Sofiya Lichkovaha
 */
public class Ship extends Actor
{
    private Ship icon = null;
    private boolean inPosition = false;
    private boolean sunk = false; 
    private int x = -1; 
    private int y = -1; 
    protected int blankX = -1;
    protected int blankY = -1; 
    protected int size; 

    protected void addedToWorld(World world) {
        x = getX();
        y = getY();
        if ((size % 2) == 0) {
            blankX = getX() + (size / 2);
            blankY = getY();
        }
    }

    public void act() {
        if (!((Battleship)getWorld()).hasStarted()) {
            place();
        }
    }
   
    public void place() {
        if (Greenfoot.mouseDragged(this)) {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            setLocation(mouse.getX(), mouse.getY());
        }
        if (Greenfoot.mouseDragEnded(this)) {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            setLocation(mouse.getX(), mouse.getY());
            if (isValidPosition(PlayerBlock.class)) {
                updatePosition();
                inPosition = true;
            } else {
                setLocation(x, y);
                inPosition = false;
            }
        }        
        if (Greenfoot.mouseClicked(this)) {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            if (mouse.getButton() == 3) {
                rotate();
                if (isValidPosition(PlayerBlock.class)) {
                    updatePosition();
                    inPosition = true;
                } else {
                    inPosition = false;
                }
            }
        }
    }
    
    public void rotate() {
        if (getRotation() == 0) {
            setRotation(270);
        } else {
            setRotation(0);
        }
    }

    private boolean isValidPosition(Class cls) {
        if ((size % 2) != 0) {
            if (getIntersectingObjects(cls).size() >= size && !isTouching(Ship.class)) {
                return true;
            } else {
                return false;
            }
        } else {
            if (getRotation() == 0) {
                if (getIntersectingObjects(cls).size() >= size && getOneObjectAtOffset((size / -2), 0, cls) != null && !isTouching(Ship.class)) {
                    return true;
                } else {
                    return false;
                }
            } else {
                if (getIntersectingObjects(cls).size() >= size && getOneObjectAtOffset(0, (size / 2), cls) != null && !isTouching(Ship.class)) {
                    return true;
                } else {
                    return false;
                }
            }
        }
    }

    public void sink()
    {
        if ((size % 2) != 0) {
            if (getIntersectingObjects(Hit.class).size() >= size) {
                sunk = true;
            }
        } else {
            if (getRotation() == 0) {
                if (getIntersectingObjects(Hit.class).size() >= size && getOneObjectAtOffset((size / -2), 0, Hit.class) != null) {
                    sunk = true;
                }
            } else {
                if (getIntersectingObjects(Hit.class).size() >= size && getOneObjectAtOffset(0, (size / 2), Hit.class) != null) {
                    sunk = true;
                }
            }
        }
        if (sunk) {
            changeImage();
            if (icon != null) {
                icon.changeImage();
            }
            Greenfoot.delay(80);
        }
    }

    private void updatePosition() {
        x = getX();
        y = getY();
        if ((size % 2) == 0) {
            if (getRotation() == 0) {
                blankX = getX() + (size / 2);
                blankY = getY();
            } else {
                blankX = getX();
                blankY = getY() - (size / 2);
            }
        }
    }

    private void changeImage() {
        setImage("sunkship"+size+".png");
    }

    public boolean move(int x, int y) {
        setLocation(x, y);
        if (!isValidPosition(ComputerBlock.class)) {
            setLocation(this.x, this.y);
            return false;
        } else {
            updatePosition();
            return true;
        }
    }

    public boolean isInPosition() {
        return inPosition;
    }

    public boolean isSunk() {
        return sunk;
    }

    public boolean isBlank(int x, int y) {
        return (x == blankX && y == blankY);
    }

    public void setIcon(Ship icon) {
        this.icon = icon;
    }
}
